@@ -57,6 +57,7 @@
 
 typedef struct
 {
+  guint gsource_id;
   GList *items;
 } HdLauncherTraverseData;
 
@@ -368,6 +369,9 @@ _hd_launcher_clear_page (GQuark key_id, gpointer data, gpointer user_data)
 {
   HdLauncherPage *page = HD_LAUNCHER_PAGE (data);
   hd_launcher_grid_clear (HD_LAUNCHER_GRID (hd_launcher_page_get_grid (page)));
+  /*
+  clutter_actor_destroy (CLUTTER_ACTOR (page));
+  */
 }
 
 static void
@@ -378,7 +382,7 @@ hd_launcher_populate_tree_starting (HdLauncherTree *tree, gpointer data)
 
   if (priv->current_traversal)
     {
-      g_idle_remove_by_data (priv->current_traversal);
+      g_source_remove(priv->current_traversal->gsource_id);
       hd_launcher_lazy_traverse_cleanup (priv->current_traversal);
     }
 
@@ -389,6 +393,11 @@ hd_launcher_populate_tree_starting (HdLauncherTree *tree, gpointer data)
       priv->pages = NULL;
     }
   g_datalist_init(&priv->pages);
+
+  if (hd_render_manager_get_state () == HDRM_STATE_LAUNCHER)
+    {
+      hd_render_manager_set_state (HDRM_STATE_HOME);
+    }
 }
 
 /*
@@ -483,7 +492,7 @@ hd_launcher_populate_tree_finished (HdLauncherTree *tree, gpointer data)
 
   if (priv->current_traversal)
     {
-      g_idle_remove_by_data (priv->current_traversal);
+      g_source_remove(priv->current_traversal->gsource_id);
       hd_launcher_lazy_traverse_cleanup (priv->current_traversal);
     }
   priv->current_traversal = tdata;
@@ -501,10 +510,11 @@ hd_launcher_populate_tree_finished (HdLauncherTree *tree, gpointer data)
   g_list_foreach (tdata->items, (GFunc) hd_launcher_create_page, NULL);
 
   /* Then we add the tiles to them in a idle callback. */
-  clutter_threads_add_idle_full (CLUTTER_PRIORITY_REDRAW + 20,
-                                 hd_launcher_lazy_traverse_tree,
-                                 tdata,
-                                 hd_launcher_lazy_traverse_cleanup);
+  tdata->gsource_id = clutter_threads_add_idle_full (
+                                     CLUTTER_PRIORITY_REDRAW + 20,
+                                     hd_launcher_lazy_traverse_tree,
+                                     tdata,
+                                     hd_launcher_lazy_traverse_cleanup);
 }
 
 /* handle clicks to the fake launch image. If we've been up this long the
@@ -638,7 +648,7 @@ hd_launcher_transition_app_start (HdLauncherApp *item)
                          hd_comp_mgr_get_current_screen_width (),
                          hd_comp_mgr_get_current_screen_height ()
                          - HD_COMP_MGR_TOP_MARGIN);
-  priv->launch_image = g_object_ref(app_image);
+  priv->launch_image = g_object_ref_sink(app_image);
 
   /* Try and get the current mouse cursor location - this should be the place
    * the user last pressed */
@@ -718,9 +728,7 @@ hd_launcher_transition_app_start (HdLauncherApp *item)
   if (item)
     title =  hd_launcher_item_get_local_name(HD_LAUNCHER_ITEM (item));
 
-  hd_title_bar_set_title (tbar,
-                          title,
-                          FALSE, TRUE);
+  hd_title_bar_set_loading_title (tbar, title);
 
   clutter_timeline_rewind(priv->launch_transition);
   clutter_timeline_start(priv->launch_transition);
