@@ -5,7 +5,9 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <glib.h>
+#include <clutter/clutter.h>
 #include <matchbox/core/mb-wm.h>
+#include <clutter/clutter.h>
 
 #include "hd-atoms.h"
 
@@ -24,14 +26,19 @@ unsigned long hd_util_modal_blocker_realize(MBWindowManagerClient *client,
 Bool hd_util_client_has_modal_blocker (MBWindowManagerClient *c);
 
 void hd_util_set_rotating_property(MBWindowManager *wm, gboolean is_rotating);
+void hd_util_set_screen_size_properties(MBWindowManager *wm, guint width, guint height);
 gboolean hd_util_change_screen_orientation (MBWindowManager *wm,
                                             gboolean goto_portrait);
 void hd_util_root_window_configured(MBWindowManager *wm);
+gboolean hd_util_rotate_geometry (ClutterGeometry *geo, guint scrh, guint scrw);
 
 gboolean hd_util_get_cursor_position(gint *x, gint *y);
 gboolean hd_util_client_has_video_overlay(MBWindowManagerClient *client);
 
 void hd_util_click (const MBWindowManagerClient *c);
 
+void
+hd_util_partial_redraw_if_possible(ClutterActor *actor, ClutterGeometry *bounds);
+
 
 #endif
